<?php
namespace Teknisa\CMP\Modules\Fornecimento\Modules\AutorizacaoFornecimento\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\Controller;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Doctrine\ORM\EntityManager;
use Exception;
use Teknisa\CMP\Controllers\ApprovalWorkflowController;

class Cmp04200LiberacaoAF extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "gerente";
    protected $entityManager;
    protected $crudService;
    protected $service;
    protected $workflow;
    
    public function __construct($crudService, ParameterBag $parameterBag, $environment, $connection, EntityManager $entityManager, $service, ApprovalWorkflowController $workflow) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->service = $service;
        $this->workflow = $workflow;
        
        parent::__construct($crudService, $parameterBag);
        
    }

    public function verificaIDAFCOMPLIMGER(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $query = "SELECT NVL(PARASUPRI.IDAFCOMPLIMGER, 'N') IDAFCOMPLIMGER, NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL) CDFILCENTCMP FROM PARASUPRI, FILIAL WHERE FILIAL.CDFILIAL = :CDFILIAL AND PARASUPRI.CDFILIAL = NVL(FILIAL.CDFILCENTCMP, FILIAL.CDFILIAL)";
        $params = array(
            "CDFILIAL" => $row['CDFILDEST']
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        $a = $stmt->fetchAll();
        $response->addDataSet(new DataSet('IDAFCOMPLIMGER', $a));
    }
    
    public function liberar(DTO\Request $request, DTO\Response $response)
    {
        $rows = $request->getDataSet()->getRows();
        
        $utilizaWorkflow = $this->utilizaWorkflow($rows[0]);
        
        $afsPendentesWorkflow = 0;
        
        foreach ($rows as $af) {
            if ($af['IDAFFORALIM'] === 'S') {
                $this->liberaIDAFFORALIM($af);
            }
            
            if ($af['IDSITUATUAAF'] === '1') {
                $this->liberaIDSITUATUAAF($af);
            }
            
            if ($utilizaWorkflow) {
                $this->service->deletarAprovacoesAlcadasExistentes($af);
                
                $situacaoWorkflow = $this->acionaWorkflow($af);
                
                if ($situacaoWorkflow == 'PENDENTE') {
                    $afsPendentesWorkflow++;
                }
            }
        }
        
        if ($afsPendentesWorkflow > 0) {
            $response->addMessage(new Message('Notificação de e-mail da Aprovação de Alçada enviada ao responsável.'));
        }
    }
    
    public function liberaIDAFFORALIM ($row) {
        $query = "UPDATE AUTOFORNE SET IDAFFORALIM = 'N', CDGERENTE = :CDGERENTE WHERE CDFILDEST = :CDFILDEST AND CDFILLANC = :CDFILLANC AND NRAUTOFORN = :NRAUTOFORN";
        $params = array(
            "CDGERENTE"  => $row['CDGERENTE'],
            "CDFILDEST"  => $row['CDFILDEST'],
            "CDFILLANC"  => $row['CDFILLANC'],
            "NRAUTOFORN" => $row['NRAUTOFORN']
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
    
    public function liberaIDSITUATUAAF ($row) {
        $query = "UPDATE AUTOFORNE SET IDSITUATUAAF = NULL, CDGERENTE = :CDGERENTE WHERE CDFILDEST = :CDFILDEST AND CDFILLANC = :CDFILLANC AND NRAUTOFORN = :NRAUTOFORN";
        $params = array(
            "CDGERENTE"  => $row['CDGERENTE'],
            "CDFILDEST"  => $row['CDFILDEST'],
            "CDFILLANC"  => $row['CDFILLANC'],
            "NRAUTOFORN" => $row['NRAUTOFORN']
        );
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
    }
    
    private function utilizaWorkflow($row)
    {
        $idUtilizaWorkflow = $this->service->getIdUtilizaWorkflow($row)['IDUTLWORKFLOW'];
        
        return $idUtilizaWorkflow == 'S';
    }
    
    public function verificaUtilizaWorkflow(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $idUtilizaWorkflow = $this->service->getIdUtilizaWorkflow($row);
        
        $response->addDataSet(new DataSet('data', $idUtilizaWorkflow));
    }
    
    private function acionaWorkflow($af)
    {
        $row = [
            'CDFILLANC'   => $af['CDFILLANC'],
            'CDFILDEST'   => $af['CDFILDEST'],
            'NRDEMANDA'   => $af['NRAUTOFORN'],
            'OPERACAO'    => 'LANCAMENTO',
            'IDTPDEMANDA' => 'AF',
        ];
        
        $customRequest = new DTO\Request\Row($row, 'POST', 'Cmp04200LiberacaoAF', 'Cmp04200LiberacaoAF');
        $customResponse = new DTO\Response();
        
        $this->workflow->checkApprovalWorkflow($customRequest, $customResponse);
        
        $messages = (array) $customResponse->getMessages();
        
        $situacaoWorkflow = array_shift($messages);
        
        return $situacaoWorkflow->getMessage();
    }
    
    public function enviarEmailParaOperadoresResponsaveisPelaAf(DTO\Request $request, DTO\Response $response){
        
        $row = $request->getRow();
        $responsilbleOperators = self::obtemOperadoresResponsaveisPelaAf($row);
        $row["OPERADORES_RESPONSAVEIS"] = $responsilbleOperators;
        
        $this->service->enviarEmailParaOperadoresResponsaveisPelaAf($row);
        
    }
    public function obtemOperadoresResponsaveisPelaAf($row){
        $selectParams = array(
            "P_CDOPERADOR"     => $row[0]["COD_COMPRADOR"],
            "P_CDFILDEST"      => $row[0]["CDFILDEST"]
  
        );
        $operators = $this->service->obtemOperadoresResponsaveisPelaAf($selectParams);
        return $operators;
        
    }
    public function confirmaSenhaGerente(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $senha = $this->service->senhaGerente($row);

            if((md5($row["SENHA"]) != $senha))
                throw new Exception("Senha inválida!");

        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }

}