/* Descrio: A funo retorna um nmero arredondado de acordo com as regras de arredondamento na Numerao Decimal da ABNT */
/* Regras de Arredondamento:
   1. Quando o algarismo imediatamente seguinte ao ltimo algarismo a ser conservado for inferior a 5, o ltimo algarismo a ser
      conservado permanecer sem modificao;
   2. Quando o algarismo imediatamente seguinte ao ltimo algarismo a ser conservado for superior a 5, ou, sendo 5, for seguido
      de no mnimo um algarismo diferente de zero, o ltimo algarismo a ser conservado dever ser aumentado de uma unidade.
   3. Quando o algarismo imediatamente seguinte ao ltimo algarismo a ser conservado for 5 seguido de zeros, dever-se- arredondar
      o algarismo a ser conservado para o algarismo par mais prximo. Conseqentemente, o ltimo algarismo a ser retirado, se for
      mpar, aumentar uma unidade.
   4. Quando o algarismo imediatamente seguinte ao ltimo a ser conservado for 5 seguido de zeros, se for par o algarismo a ser
      conservado, ele permanecer sem modificao.*/

CREATE OR REPLACE FUNCTION ROUND_ABNT(
   P_VALOR         IN VARCHAR,
   P_CASASDECIMAIS IN VARCHAR) RETURN NUMBER
AS
   V_DECIMAIS         VARCHAR2(250);
   V_ULTIMOS          VARCHAR2(250);
   V_SUB              CHAR(1);
   V_DIGITOCONSERVADO CHAR(1);
   V_SEPARADOR        CHAR(1);
   V_RETORNA          VARCHAR2(20);
   V_AJUSTA           NUMBER;
BEGIN
   /*Posio do Separador no Nmero*/
   V_SEPARADOR        := INSTR(P_VALOR, TO_CHAR(0, 'fmd'));
   /*Se for um nmero inteiro no  necessrio arredondar*/
   IF V_SEPARADOR = 0 THEN
      V_RETORNA := P_VALOR;
   ELSE
      /*Verifica o tamanho das casas decimais, para saber em quantas posies o nmero deve ser ajustado*/
      IF LENGTH(SUBSTR(P_VALOR, V_SEPARADOR + 1)) > P_CASASDECIMAIS + 2 THEN
         V_AJUSTA := LENGTH(SUBSTR(P_VALOR, V_SEPARADOR + 1));
      ELSE
         V_AJUSTA := P_CASASDECIMAIS + 2;
      END IF;
      /*Casas Decimais*/
      V_DECIMAIS         := RPAD(SUBSTR(P_VALOR, V_SEPARADOR + 1),V_AJUSTA,0);
      /*Segundo Dgito aps o separador*/
      V_ULTIMOS          := SUBSTR(V_DECIMAIS, P_CASASDECIMAIS + 2);
      /*Primeiro Dgito aps o separador*/
      V_SUB              := SUBSTR(V_DECIMAIS, P_CASASDECIMAIS + 1, 1);
      /*ltimo Dgito a ser Conservado, pela quantidade de casas decimais*/
      V_DIGITOCONSERVADO := SUBSTR(V_DECIMAIS, P_CASASDECIMAIS, 1);

      IF V_SUB < 5 THEN
          V_RETORNA := TRUNC(TO_NUMBER(P_VALOR), P_CASASDECIMAIS);
      ELSIF V_SUB > 5 OR (V_SUB = 5 AND V_ULTIMOS > 0) THEN
          V_RETORNA := SUBSTR(P_VALOR, 0, V_SEPARADOR - 1)||TO_CHAR(0, 'fmd')||(SUBSTR(V_DECIMAIS, 0, P_CASASDECIMAIS)) + ('0'||TO_CHAR(0, 'fmd')||LPAD('1',P_CASASDECIMAIS,0));
      ELSIF V_SUB = 5 AND V_ULTIMOS = 0 AND MOD(V_DIGITOCONSERVADO, 2) = 0 THEN
          V_RETORNA := TRUNC(TO_NUMBER(P_VALOR), P_CASASDECIMAIS);
      ELSIF V_SUB = 5 AND V_ULTIMOS = 0 THEN
          V_RETORNA := SUBSTR(P_VALOR, 0, V_SEPARADOR - 1)||TO_CHAR(0, 'fmd')||(SUBSTR(V_DECIMAIS, 0, P_CASASDECIMAIS)) + ('0'||TO_CHAR(0, 'fmd')||LPAD('1',P_CASASDECIMAIS,0));
      END IF;
   END IF;

   RETURN V_RETORNA;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('ROUND_ABNT','6.19.000'); END;
/
